#pragma once
/// @file LayoutUIClient.h
#include "common.h"

enum LVM_TYPE { LVM_INVALID, LVM_POINT, LVM_AREA };
const int LVM_FLAG_AUTODRAW = 1; // Allow the framework to draw this manipulator
const int LVM_FLAG_SIMPLE = 2;	 // Actions passed to this manipulator will only be of type LVM_SIMPLE

struct LV_Manipulator {
	LV_Manipulator() { type = LVM_INVALID; flags = 0; code = -1; };
	LVM_TYPE type;
	int code;			// Manipulator ID code, to be interpreted by the LayoutUIClient as it sees fit
	Vector2 point;
	Rect4F area;
	int flags;
};

enum LVM_ACTION { LVM_SIMPLE,	// Simple action
				  LVM_LBDOWN, LVM_LBUP, LVM_RBDOWN, LVM_RBUP, LVM_DRAG, LVM_DBLCLICK };	// Complex actions

const int LVM_FLAG_SHIFT	= 1;
const int LVM_FLAG_CTRL		= 2;
const int LVM_FLAG_ALT		= 4;

enum LVUI_STYLE { LVUI_NORMAL, LVUI_HOT, LVUI_LIT, LVUI_SELECTED };

struct LV_GUIData {
	LVUI_STYLE style;
	bool flatten_shapes;
	float scale;
// Suggested colors
	DWORD base_color;
	DWORD line_color;
	DWORD outer_color;
	DWORD knot_color;
	int hot_code;
};


class Device;
enum LUI_KNOB_TYPE { LUI_BASIC_KNOB, LUI_CROSS };

/** Layout Mode UI Client class. 

	This class in fact belongs to the World Machine Application.

	It is in the core as many elements of World Machine can potentially expose UI elements and need to do so
	in a predictable way.

	Much of this is still undeveloped functionality and so is flagged as TODO
	@ingroup application
*/
class LayoutUIClient
{
public:
	LayoutUIClient();
	virtual ~LayoutUIClient(void);

	virtual char *getName();
	virtual int getType() { return 'BASE'; };

/// override these virtual functions to change the way the framework draws/interacts with your UI
	virtual bool queryAutoDraw() { return true; };	// Return true if you want any of your manipulators automatically drawn

/// @name OpenGL Calls
/// The current GL context will be bound when these calls are made, and ONLY when these calls are made.
/// allowing you to manage GL resources (textures, etc) and draw to the context.
//@{
	/// Perform any needed initialization (create textures, etc)
	virtual bool OnCreate() { return true; };
	/// Shutdown any GL-related state you've created
	virtual bool OnDestroy() { return true; };

	/// Draw any special needs you have. Manipulators will be automatically drawn if set to LVM_FLAG_AUTODRAW, so you may not
	/// have to override this
	virtual bool OnDraw(LV_GUIData &style) { return false; };
//@}

/** @name Manipulators
	A Manipulator is an exposed hotspot that can be manipulated with the mouse by the user in the application. 
	The GL state is NOT bound during these calls 
@{
	*/
	virtual int				numManipulator() { return 0; };
	virtual char*			getManipulatorHelp(int i) { return NULL; };	// Return help string for this manipulator
	virtual LV_Manipulator  getManipulator(int i) { LV_Manipulator empty; return empty; };
	virtual bool			actionManipulator(int i, LVM_ACTION act, Vector2 wpoint, int flags) { return false; };
//@}
};

/// Device to Layout Mode (OpenGL UI) link interaction class
/// If your device wants Layout Mode overview-mode interactors, subclass from this interface,
/// place in your Device class, and return a pointer to it from your derived version of Device::GetLayoutUI()
/// @ingroup application devices
class DeviceLayoutUIClient : public LayoutUIClient {
public:
	DeviceLayoutUIClient();
	virtual ~DeviceLayoutUIClient();

	virtual char *getName();
	virtual int getType() { return 'DEVI'; };

	void setOwner(Device *_owner) { base_owner = _owner; };
	Device *getOwner() { return base_owner; };
protected:
	Device *base_owner;
};