#if !defined(AFX_WMPACKET_H__90A4604E_7B5D_4668_AA28_ECCC0ABCCA34__INCLUDED_)
#define AFX_WMPACKET_H__90A4604E_7B5D_4668_AA28_ECCC0ABCCA34__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/** @file WMPacket.h

	Class definition for WMPacket, plus other helper functions and defines

*/

#include "common.h"
#include "globals.h"
#include "Context.h"

/// @name Packet Types
/// @relates WMPacket
//@{
const int WMP_ANY		= 0;
const int WMP_parameter	= 1;			// Parameter Packet
const int WMP_scalar	= 2;
const int WMP_hfield	= 4;			// Heightfields
const int WMP_rgb		= 8;			// RGB Bitmaps
const int WMP_volfield	= 16;
const int WMP_coord		= 32;			// Coordinate Packet
const int WMP_text		= 64;			// Text
const int WMP_combo		= 128;
const int WMP_mesh		= 256;			// Mesh
const int WMP_areabase	= 512;			// WMAreaPacket-derived class
const int WMP_gridbase = 1024;			// WMGridPacket-derived class
const int WMP_reserved8 = 2048;
const int WMP_reserved9 = 4096;
const int WMP_reserved10= 8192;
const int WMP_PARAMS	= WMP_parameter | WMP_scalar;
// If you want to create new, user-defined packet types, CONTACT ME FIRST.
// I will give you the bitcode to use to prevent collisions with any other packet type

/// Return a string with type(s) of packet by ID
std::string getPacketTypeString(const int type, const std::string concat_str = ", ");
//@}

/// @name Packet Display Hints. 
/// Only used for visual purposes, does not effect underlying treatment of data
//@{
const int WMP_hint_landscape = 0; // treat as a landscape
const int WMP_hint_mask		 = 1; // treat as a mask
const int WMP_hint_nolight   = 2; // do not apply lighting; data may be prelit
//@}

/** WMPacket is the base class that encapsulates information transfer between
  machines. 
  
  Most common subclass types are HField and BMPacket (heightfield and RGB bitmap respectively)

  However any information that needs be passed between devices can be derived from
  WMPacket, even allowing for User-defined information transfer.

  Note that the application display classes may not know what to do with any custom class you create.

	Also Note that packets should *NOT* be created on the stack (locally). You must acquire them
	through the global factory method this ensures that the proper heap is being used for memory alloc across DLLs.

 To create a packet, you must use the factory function in WMGlobal. All derived classes include
 a static GetXXX() function that simply wraps this call for convenience.



  @ingroup data
  @ingroup memory

*/
class WMPacket  
{
public:

/** @name Lifecycle Methods
 To create a packet, you must use the factory function in WMGlobal. All derived classes include
 a static GetXXX() function that simply wraps this call for convenience
*/
//@{
/// Return a copy of this WMPacket-derived packet
	virtual PacketPointer Clone() const = 0;			
/// Assignment is allowed as it doesnt involve creation or destruction
	WMPacket &operator=(const WMPacket &rhs);
	int getRefCount() const { return ref_count; };	///< Current reference count

	virtual void Clear() = 0;					///< Basic initialization of this packet to a default state
//@}

	/// @name Type System
	//@{
	virtual int getType() = 0;				///< Retrieve datatype ID of this packet
	virtual char *getTypeName() = 0;		///< Retrieve human-readable name of this type of data
	virtual bool queryType(const int type);	///< Query if this packet allows for a given type interface. 
	//@}

	/// @name Serialization
	//@{
	virtual bool canSerialize() = 0;				///< Can this type of data be saved/loaded from disk?
	virtual bool Load(std::istream &input) = 0;		///< Load from stream input
	virtual bool Save(std::ostream &output) = 0;	///< Save a copy of this data to stream output
	//@}

	/// @name UI-related
	//@{
	virtual dword Color() = 0;						///< Returns a 32bit color for this type of data. (visualization purposes only) 
	int getDisplayHint() const { return hint; };	///< Hints to WM how to display this HF. 
	void setDisplayHint(int _hint) { hint = _hint; };
	void SetNameTag(const char *name);				///< Tag this packet with a name
	const char *GetNameTag() const;					///< Return packet's name
	//@}

	/// @name Memory Management
	//@{
	virtual size_t getHeapUsage() = 0;				///< Return the number of bytes of WM heap memory this packet is using
	/// Can this packet be paged out to disk currently?
	bool isPageable() { return page_able && (page_views == 0); };	
	/// Is this packet paged out to disk?
	bool isPaged();
	bool PageIn();									///< Bring the packet back in from disk, if it has been paged out.
	bool PageOut(bool clearcontents = true);		///< Page this packet to disk, IF it is pageable. Optionally empty its contents after paging
	bool setPagable(bool available);				///< Set page-ability of the packet. Setting this to true means that its data can be paged out at any time by the core.
	void acquireAccessible();						///< Call when you need to temporarily read from a possibly-paged-out packet
	void releaseAccessible();						///< Call when done reading from said packet

	void setPageProtector(CRITICAL_SECTION *crit);	///< Pass in an external critical section that this packet will synchronize on when it needs to page in or out.
	//@}

protected:
#define NAMELEN		32

/// @name Protected constructor/destructor, use WM core only to allocate and delete
//@{
	friend class WMPacketFactory;
	WMPacket();
	virtual ~WMPacket();
	virtual void doKill();		// Call to delete its own type
//@}


	virtual void resetContents() = 0;					///< Called to reset contents during page out operation
	virtual bool isTypePageable() { return false; };	///< Intrinsic to the class of packet, if this should be page-able or not.

	char nametag[NAMELEN];	///< nametag for this packet
	int hint;
private:
	WMPacket(const WMPacket &other);		///< no copy construction
	void private_deletepaging();

	CRITICAL_SECTION *page_crit;			// pointer to critical section protecting this packets paging mechanism. May be NULL if packet is not participating in paging.
	volatile bool page_state;				// if packet is currently paged out
	volatile bool page_able;				// CAN the packet be paged out?
	volatile unsigned int  page_views;		// number of views 
	char page_swapname[NAMELEN];	// disk filename of paged data

	volatile int ref_count;			///< Reference counting for the packet
	int AddRef();									///< increment reference counter
	int Release();									///< Decrease the ref count by one, deleting if appropriate.
	friend void intrusive_ptr_add_ref(WMPacket *wmp);
	friend void intrusive_ptr_release(WMPacket *wmp);
};

extern void intrusive_ptr_add_ref(WMPacket *wmp);
extern void intrusive_ptr_release(WMPacket *wmp);

/** Abstract WMPacket-derived base class that coverts some "area" of space
*/
class WMAreaPacket : public WMPacket  {
public:
	virtual WMAreaPacket &operator=(const WMAreaPacket &rhs);

	virtual void Init(const PacketContext &_size);
	virtual bool queryType(const int type);

/// @name Area/Size functions
//@{
	/// Retrieve context reference for modifications
	PacketContext &getContext() { return areacontext; };
	/// Return const PacketContext
	const PacketContext &getContext() const { return areacontext; };
	/// width of heightfield
	inline int w() const { return areacontext.w(); };
	/// height of heightfield
	inline int h() const { return areacontext.h(); };
	/// area of heightfield (w*h)
	inline size_t area() const { return areacontext.resarea(); };
	/// size returned as a Coord (x,y) pair
	inline Coord getsize() const { return areacontext.getRes(); };
	/// *worldsize* returns as a floating point pair
	inline CoordF getRegionSize() const { return areacontext.getRegionSize(); };

	inline float getDScale() const { return areacontext.getDScale(); };
	inline float getVScale() const {  return areacontext.getVscale(); };
//@}
protected:
	WMAreaPacket();
	virtual ~WMAreaPacket();
	friend class WMPacketFactory;

	PacketContext areacontext;
};

typedef void(visitor_type)(Coord);

/*
inline WMAreaPacket *AREAPACKET(PacketPointer pack) { 
	if (pack) {
		bool ok = pack->getType() == WMP_hfield;
		if (!ok)
			return NULL;
	}
	return HFPointer((HField*) pack.get()); 
};
*/

#endif // !defined(AFX_WMPACKET_H__90A4604E_7B5D_4668_AA28_ECCC0ABCCA34__INCLUDED_)
